﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Windows;
using System.Threading;

namespace AyaClock.NET
{
    /// <summary>
    /// App.xaml の相互作用ロジック
    /// </summary>
    public partial class App : Application
    {
        // 多重起動禁止のロジックは
        //   http://akabeko.sakura.ne.jp/blog/2009/10/wpf-%E3%82%A2%E3%83%97%E3%83%AA%E3%81%AE%E8%B5%B7%E5%8B%95%E5%87%A6%E7%90%86/
        // を参考にした

        /// <summary>
        /// 多重起動禁止用mutex
        /// </summary>
        private static Mutex _mutex;

        /// <summary>
        /// アプリケーションが開始される時のイベント。
        /// </summary>
        protected override void OnStartup(StartupEventArgs e)
        {
            App._mutex = new Mutex(false, "AyaClock.NET2009.mutex");
            if (!App._mutex.WaitOne(0, false))
            {
                App._mutex.Close();
                App._mutex = null;
                MessageBox.Show("AyaClock.NET already exists", "AyaClock.NET");
                this.Shutdown();
                return;
            }

            // show main window
            Window1 window = new Window1();
            window.Show();
        }

        /// <summary>
        /// アプリケーションが終了する時のイベント。
        /// </summary>
        protected override void OnExit(ExitEventArgs e)
        {
            if (App._mutex == null) { return; }

            // release mutex
            App._mutex.ReleaseMutex();
            App._mutex.Close();
            App._mutex = null;
        }

    }
}
