﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Threading;
using System.Security.Principal;
using System.Runtime.InteropServices;
using VistaBridge.UI;

namespace AyaClock.NET
{
    /// <summary>
    /// Window1.xaml の相互作用ロジック
    /// </summary>
    public partial class Window1 : Window
    {
        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        private struct MEMORYSTATUSEX
        {
            public UInt32 dwLength;
            public UInt32 dwMemoryLoad;
            public UInt64 ullTotalPhys;
            public UInt64 ullAvailPhys;
            public UInt64 ullTotalPageFile;
            public UInt64 ullAvailPageFile;
            public UInt64 ullTotalVirtual;
            public UInt64 ullAvailVirtual;
            public UInt64 ullAvailExtendedVirtual;
        }

        [DllImport("kernel32.dll", SetLastError = true)]
        private static extern bool GlobalMemoryStatusEx(ref MEMORYSTATUSEX lpBuffer);

        /// <summary>
        /// 表示設定用プロパティ群
        /// </summary>
        private bool _StayOnTop;
        private bool StayOnTop {
            get { return _StayOnTop; }
            set
            {
                if (this._StayOnTop == value ) return ;
                this._StayOnTop = value;
                this.Topmost = this._StayOnTop;
                ayaclockconfig.Instance.StayOnTop = this._StayOnTop;
                ayaclockconfig.SaveConfig();
            }
        }

        private bool _IgnoreTaskBar;
        private bool IgnoreTaskBar
        {
            get { return _IgnoreTaskBar; }
            set
            {
                if (this._IgnoreTaskBar == value) return;
                this._IgnoreTaskBar = value;
                FormUtils.AdjustWindowPos(this, PosType, this._IgnoreTaskBar);
                ayaclockconfig.Instance.IgnoreTaskBar = this._IgnoreTaskBar;
                ayaclockconfig.SaveConfig();
            }
        }

        private PositionType _PosType;
        private void PosMnChecked(MenuItem CheckedMn)
        {
            LeftTopMn.IsChecked = LeftTopMn.Equals(CheckedMn);
            LeftBottomMn.IsChecked = LeftBottomMn.Equals(CheckedMn);
            RightTopMn.IsChecked = RightTopMn.Equals(CheckedMn);
            RightBottomMn.IsChecked = RightBottomMn.Equals(CheckedMn);
            FreedomMn.IsChecked = FreedomMn.Equals(CheckedMn);
        }
        private PositionType PosType
        {
            get { return this._PosType; }
            set
            {
                if (this._PosType != value)
                {
                    this._PosType = value;
                    FormUtils.AdjustWindowPos(this, PosType,IgnoreTaskBar);
                    switch (PosType)
                    {
                        case PositionType.LeftTop:
                            PosMnChecked(LeftTopMn);
                            break;
                        case PositionType.LeftBottom:
                            PosMnChecked(LeftBottomMn);
                            break;
                        case PositionType.RightTop:
                            PosMnChecked(RightTopMn);
                            break;
                        case PositionType.RightBottom:
                            PosMnChecked(RightBottomMn);
                            break;
                        case PositionType.Any:
                            PosMnChecked(FreedomMn);
                            break;
                    }
                    ayaclockconfig.Instance.PosType = this._PosType;
                    if (PosType == PositionType.Any)
                    {
                        ayaclockconfig.Instance.Left = (int)Left;
                        ayaclockconfig.Instance.Top = (int)Top;
                    }
                    ayaclockconfig.SaveConfig();
                }
            }
        }

        private MemUnitType MemUnit
        {
            get
            {
                if (KByteMn.IsChecked)
                    return MemUnitType.KByte;
                else if (MByteMn.IsChecked)
                    return MemUnitType.MByte;
                else
                    return MemUnitType.GByte;
            }
            set
            {
                if (value != this.MemUnit)
                {
                    switch (value)
                    {
                        case MemUnitType.KByte:
                            KByteMn.IsChecked = true;
                            MByteMn.IsChecked = false;
                            GByteMn.IsChecked = false;
                            break;
                        case MemUnitType.MByte:
                            KByteMn.IsChecked = false;
                            MByteMn.IsChecked = true;
                            GByteMn.IsChecked = false;
                            break;
                        case MemUnitType.GByte:
                            KByteMn.IsChecked = false;
                            MByteMn.IsChecked = false;
                            GByteMn.IsChecked = true;
                            break;
                    }
                    ayaclockconfig.Instance.MemUnit = value;
                    ayaclockconfig.SaveConfig();
                }
            }
        }

        private bool ShowComma
        {
            get { return CommaMn.IsChecked; }
            set 
            {
                if (value != this.ShowComma)
                {
                    CommaMn.IsChecked = value;
                    ayaclockconfig.Instance.ShowComma = value;
                    ayaclockconfig.SaveConfig();
                }
            }
        }

        private void SetWeekdayMn(MenuItem ChkMn)
        {
            WeekDayNoneMn.IsChecked = WeekDayNoneMn.Equals(ChkMn);
            En2ChMn.IsChecked = En2ChMn.Equals(ChkMn);
            En3ChMn.IsChecked = En3ChMn.Equals(ChkMn);
            KanjiMn.IsChecked = KanjiMn.Equals(ChkMn);
            OsWeekdayMn.IsChecked = OsWeekdayMn.Equals(ChkMn);
        }
        private WeekdayType Weekday
        {
            get
            {
                if (En2ChMn.IsChecked)
                    return WeekdayType.Eng2ch;
                if (En3ChMn.IsChecked)
                    return WeekdayType.Eng3ch;
                if (KanjiMn.IsChecked)
                    return WeekdayType.Kanji;
                if (OsWeekdayMn.IsChecked)
                    return WeekdayType.OsSetting;
                return WeekdayType.None;
            }
            set
            {
                if (value != this.Weekday)
                {
                    switch (value)
                    {
                        case WeekdayType.None:
                            SetWeekdayMn(WeekDayNoneMn);
                            break;
                        case WeekdayType.Eng2ch:
                            SetWeekdayMn(En2ChMn);
                            break;
                        case WeekdayType.Eng3ch:
                            SetWeekdayMn(En3ChMn);
                            break;
                        case WeekdayType.Kanji:
                            SetWeekdayMn(KanjiMn);
                            break;
                        case WeekdayType.OsSetting:
                            SetWeekdayMn(OsWeekdayMn);
                            break;
                    }
                    ayaclockconfig.Instance.Weekday = value;
                    ayaclockconfig.SaveConfig();
                }
            }
        }

        private bool ShowSecond
        {
            get { return SecondMn.IsChecked; }
            set
            {
                if (value != this.ShowSecond)
                {
                    SecondMn.IsChecked = value;
                    ayaclockconfig.Instance.ShowSecond = value;
                    ayaclockconfig.SaveConfig();
                }
            }
        }

        private bool ShowTasktrayIcon
        {
            get { return ni.Visible; }
            set
            {
                if (value != this.ShowTasktrayIcon)
                {
                    TasktrayMn.IsChecked = value;
                    ni.Visible = value;
                    ayaclockconfig.Instance.ShowTasktrayIcon = value;
                    ayaclockconfig.SaveConfig();
                }
            }
        }

        /// <summary>
        ///  内部処理用プロパティなどなど
        /// </summary>

        private string MemUnitStr
        {
            get
            {
                switch(MemUnit)
                {
                    case MemUnitType.KByte:
                        return "KB";
                    case MemUnitType.MByte:
                        return "MB";
                    case MemUnitType.GByte:
                        return "GB";
                    default:
                        return "MB";
                }
            }
        }

        private static string[,] WeekdayStrs = { /* none */  { "","","","","","","" },
                                                 /* Eng3 */  { "(Sun)","(Mon)","(Tue)","(Wed)","(Thu)","(Fri)","(Sat)" },
                                                 /* Eng2 */  { "(Su)","(Mo)","(Tu)","(We)","(Th)","(Fr)","(Sa)" },
                                                 /* Kanji */ { "(日)","(月)","(火)","(水)","(木)","(金)","(土)" },
                                                 /* OS */    { "","","","","","","" } };
        private string WeekdayStr
        {
            get { return WeekdayStrs[(int)Weekday, int.Parse(DateTime.Now.DayOfWeek.ToString("d"))]; }
        }

        private UInt64 MemModifier
        {
            get
            {
                switch (MemUnit)
                {
                    case MemUnitType.KByte:
                        return 1024;
                    case MemUnitType.MByte:
                        return 1024 * 1024;
                    case MemUnitType.GByte:
                        return 1024 * 1024 * 1024;
                        break;
                    default:
                        return 1024 * 1024;
                }
            }
        }

        private System.Globalization.NumberFormatInfo nfi = null;
        private string ToCommaStr(UInt64 AValue)
        {
            if (nfi == null)
            {
                nfi = (System.Globalization.NumberFormatInfo)System.Globalization.NumberFormatInfo.CurrentInfo.Clone();
                nfi.NumberDecimalDigits = 0;
            }
            return AValue.ToString("N", nfi);
        }

        /// <summary>
        /// 日付・時間・空きメモリのupdate
        /// </summary>
        
        private int tPassed = -1;

        private void SetOsWeekdayStrs(DateTime N)
        {
            // 曜日文字列設定 - 短い曜日文字列
            int cnt = 0;
            while (cnt < 7)
            {
                WeekdayStrs[4, cnt] = "("+System.Globalization.DateTimeFormatInfo.CurrentInfo.ShortestDayNames[cnt]+")";
                cnt++;
            }
        }

        private void UpdateTimeAndMem()
        {
            // 最初に時間を保存
            DateTime N = DateTime.Now;
    
            // 日付
            string MDfmt = DateTimeFormatInfo.CurrentInfo.ShortDatePattern.ToUpper(); // 短い日付フォーマット
            string MDsep = DateTimeFormatInfo.CurrentInfo.DateSeparator;
            string work = "";

            int Mpos = MDfmt.IndexOf('M');
            int Dpos = MDfmt.IndexOf('D');

            if (Mpos==-1 || Dpos==-1 || Mpos<Dpos)
                // 月/日の場合 or なんかエラーだった場合
                work = string.Format("{0:00}"+MDsep+"{1:00}",
                                     int.Parse(N.Month.ToString()),
                                     int.Parse(N.Day.ToString()));
            else
                // 日/月の場合
                work = string.Format("{0:00}"+MDsep+"{1:00}",
                                     int.Parse(N.Day.ToString()),
                                     int.Parse(N.Month.ToString()));
            // 曜日
            DateLB.Content = work + WeekdayStr;
            
            // 時間
            if (ShowSecond)
                TimeLB.Content = N.TimeOfDay.ToString().Substring(0, 8);
            else
                TimeLB.Content = N.TimeOfDay.ToString().Substring(0, 5);

            // 空きメモリ
            UInt64 m = 0;
            MEMORYSTATUSEX memsts = new MEMORYSTATUSEX();
            memsts.dwLength = (UInt32)System.Runtime.InteropServices.Marshal.SizeOf(memsts);
            if (GlobalMemoryStatusEx(ref memsts))
                m = memsts.ullAvailPhys / MemModifier;
            if (ShowComma)
                MemLB.Content = ToCommaStr(m) + MemUnitStr;
            else
                MemLB.Content = m.ToString() + MemUnitStr;

            // 更新「分秒」保存
            tPassed = N.Minute*100+N.Second;
        }

        // ウインドウ移動用ワーク
        private Point sv_pt;

        // コンストラクタ
        public Window1()
        {
            InitializeComponent();
            InitDatas();
        }

        // デストラクタ
        ~Window1()
        {
            if (ni!=null) ni.Dispose();
        }

        // 各種初期化処理
        private void FontSettingForLabel(ref Label ALabel, string brushcolor)
        {
            ALabel.FontFamily = label1.FontFamily;
            ALabel.FontSize = label1.FontSize;
            BrushConverter bc = new BrushConverter();
            ALabel.Foreground = (System.Windows.Media.Brush)bc.ConvertFrom(brushcolor);
            ALabel.FontWeight = label1.FontWeight;
            ALabel.FontStyle = label1.FontStyle;
        }

        private void FontSettingForLabels(string brushcolor)
        {
            FontSettingForLabel(ref DateLB, brushcolor);
            FontSettingForLabel(ref TimeLB, brushcolor);
            FontSettingForLabel(ref MemLB, brushcolor);
        }

        private void SetSmallShieldIcon()
        {
            // XPならば無し
            System.OperatingSystem os = System.Environment.OSVersion;
            if (os.Version.Major < 6) return;
            // Vista以降ならばAdjustTime Menuにシールドアイコンを設定
            StockIcon icon = new StockIcon();
            icon.Identifier = StockIconIdentifier.Shield;
            icon.Small = true;
            Image image = new Image();
            image.Source = icon.Bitmap;
            image.Stretch = Stretch.None;
            AdjustTimeMn.Icon = image;
        }

        // PopupMenu表示
        private void ShowPopupMenu()
        {
            this.Show();
            this.PopupMenu.IsOpen = true;
        }

        // NotifyIconについては以下を参照 
        //   http://www.be-styles.jp/archives/2468
        //   http://blogs.wankuma.com/youryella/archive/2009/11/01/182630.aspx
        //   http://hogetan.blog24.fc2.com/blog-category-3.html
        private System.Windows.Forms.NotifyIcon ni = null;
        private void SetTasktrayIcon()
        {
            System.Reflection.Assembly myAssembly = System.Reflection.Assembly.GetExecutingAssembly();
            ni = new System.Windows.Forms.NotifyIcon();
            ni.Icon = new System.Drawing.Icon(myAssembly.GetManifestResourceStream("AyaClock.NET.ACLOCKS3.ICO"));
            ni.Text = "AyaClock.NET";
            ni.Visible = false;
            ni.MouseClick += 
                 delegate(object sender, System.Windows.Forms.MouseEventArgs e)
                 {
                     if (e.Button == System.Windows.Forms.MouseButtons.Right)
                     {
                         this.ShowPopupMenu();
                     }
                 };
        }

        private void InitDatas()
        {
            LeftTopMn.IsChecked = true;
            // shield iconの設定
            SetSmallShieldIcon();
            // Tasktary Iconの設定
            SetTasktrayIcon();
            // 曜日文字列の初期設定
            SetOsWeekdayStrs(DateTime.Now);
            // configure data読み込み
            ayaclockconfig.LoadConfig();
            // Tasktray Icon表示の有無
            ShowTasktrayIcon = ayaclockconfig.Instance.ShowTasktrayIcon;
            // メモリ表示
            MByteMn.IsChecked = true;
            MemUnit = ayaclockconfig.Instance.MemUnit;
            ShowComma = ayaclockconfig.Instance.ShowComma;
            // 曜日表示
            WeekDayNoneMn.IsChecked = true;
            Weekday = ayaclockconfig.Instance.Weekday;
            // 秒表示
            SecondMn.IsChecked = false;
            ShowSecond = ayaclockconfig.Instance.ShowSecond;
            // フォント
            FontFamilyConverter ffc = new FontFamilyConverter();
            label1.FontFamily = (System.Windows.Media.FontFamily)ffc.ConvertFromString(ayaclockconfig.Instance.FontName);
            label1.FontSize = ayaclockconfig.Instance.FontSize;
            string argb = "#" + Convert.ToString(ayaclockconfig.Instance.Color, 16);
            //BrushConverter bc = new BrushConverter();
            //label1.Foreground = (System.Windows.Media.Brush)bc.ConvertFrom(argb);
            label1.Content = "";
            if (ayaclockconfig.Instance.Bold)
                label1.FontWeight = System.Windows.FontWeight.FromOpenTypeWeight(700);
            else
                label1.FontWeight = System.Windows.FontWeight.FromOpenTypeWeight(400);
            if (ayaclockconfig.Instance.Italic)
                label1.FontStyle = System.Windows.FontStyles.Italic;
            else
                label1.FontStyle = System.Windows.FontStyles.Normal;
            FontSettingForLabels(argb);
            // 設定条件からwindow size決定
            CalcSize();
            // 表示文字列の設定
            UpdateTimeAndMem();
            // 位置の復帰 - sizeが決まってから実行
            IgnoreTaskBar = ayaclockconfig.Instance.IgnoreTaskBar;
            StayOnTop = ayaclockconfig.Instance.StayOnTop;
            if (ayaclockconfig.Instance.PosType == PositionType.Any)
            {
                Left = ayaclockconfig.Instance.Left;
                Top = ayaclockconfig.Instance.Top;
            }
            PosType = ayaclockconfig.Instance.PosType; // 位置情報の後で復帰する - 位置調整のため
            // Timer Start
            DispatcherTimer timer = new DispatcherTimer();
            timer.Tick += new EventHandler(OnTimer);
            timer.Interval = new TimeSpan(0, 0, 0, 0, 200);
            timer.Start();
        }

        // Timer処理
        private void OnTimer(object sender, EventArgs e)
        {
            // まだ準備ができていない
            if (!label1.IsInitialized) return;

            // 移動中は処理停止
            if (label1.IsMouseCaptured) return;

            // 「分秒」が変わった？
            DateTime N = DateTime.Now;
            int sv = N.Minute * 100 + N.Second;
            if (tPassed != sv)
            {
                // 変わっていたので更新
                UpdateTimeAndMem();
            }

            // 必要があれば位置調整
            FormUtils.AdjustWindowPos(this, PosType, IgnoreTaskBar);
        }

        // AyaClock.NETの終了
        private void QuitMn_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        // label1からFormattedText生成
        private FormattedText MakeFT(string AText)
        {
            FormattedText result = new FormattedText(AText,
                                                     System.Globalization.CultureInfo.CurrentCulture,
                                                     FlowDirection.LeftToRight,
                                                     new Typeface(label1.FontFamily.ToString()),
                                                     label1.FontSize,
                                                     Brushes.Black);
            result.SetFontWeight(label1.FontWeight);
            result.SetFontStyle(label1.FontStyle);
            return result;
        }

        // 各種設定からWindow Size決定
        private void CalcSize()
        {
            // 数字の中で一番幅の大きい文字を取得
            FormattedText ft = MakeFT("0");
            double numWidth = ft.Width, maxH = ft.Height;
            char max_c = '0';
            for (char c = '1'; c <= '9'; c++)
            {
                ft = MakeFT(""+c);
                if (ft.Width > numWidth)
                {
                    max_c = c; numWidth = ft.Width;
                }
                if (ft.Height > maxH)
                {
                    maxH = ft.Height;
                }
            }

            // ラベル間のスペースを決定 : 一番幅の大きい数字の1/2
            double spWidth = numWidth / 2;

            // 日付ラベルの幅を決定
            string dateStr = "" + max_c + max_c + DateTimeFormatInfo.CurrentInfo.DateSeparator + max_c + max_c;
            string wdStr = WeekdayStrs[(int)Weekday, 0];
            ft = MakeFT(wdStr);
            double wdWidth = ft.Width;
            if (ft.Height > maxH)
            {
                maxH = ft.Height;
            }
            for (int i = 1; i < 7; i++)
            {
                ft = MakeFT(WeekdayStrs[(int)Weekday, i]);
                if (ft.Width > wdWidth)
                {
                    wdStr = WeekdayStrs[(int)Weekday, i];
                    wdWidth = ft.Width;
                }
                if (ft.Height > maxH)
                {
                    maxH = ft.Height;
                }
            }
            ft = MakeFT(dateStr + wdStr);
            wdWidth = ft.Width + 2 * DateLB.Padding.Left;
            if (ft.Height > maxH)
            {
                maxH = ft.Height;
            }

            // 時間ラベルの幅を決定
            string timeStr = "" + max_c + max_c + DateTimeFormatInfo.CurrentInfo.TimeSeparator + max_c + max_c;
            if (ShowSecond)
                timeStr = timeStr + DateTimeFormatInfo.CurrentInfo.TimeSeparator + max_c + max_c;
            ft = MakeFT(timeStr);
            double tmWidth = ft.Width + 2 * TimeLB.Padding.Left;
            if (ft.Height > maxH)
            {
                maxH = ft.Height;
            }

            // メモリラベルの幅を決定
            MEMORYSTATUSEX memsts = new MEMORYSTATUSEX();
            UInt64 m=0,max_m=0;
            memsts.dwLength = (UInt32)System.Runtime.InteropServices.Marshal.SizeOf(memsts);
            if (GlobalMemoryStatusEx(ref memsts)) {
                m = memsts.ullAvailPhys / MemModifier;
                max_m = memsts.ullTotalPhys / MemModifier;
            }
            UInt64 MaxLen = (UInt64)max_m.ToString().Length; // 搭載メモリの最大長
            UInt64 MaxLenNum = 0;
            for (UInt64 i = 0; i < MaxLen; i++)
            {
                // 仮の数字 1 で最大桁数の数字を作成しておく
                MaxLenNum = MaxLenNum * 10 + 1;
            }
            string memStr = "";
            if (ShowComma)
                memStr = ToCommaStr(MaxLenNum);
            else
                memStr = MaxLenNum.ToString();
            memStr.Replace('1', max_c); // 1 を 最大幅の文字で置き換える
            memStr += MemUnitStr;
            ft = MakeFT(memStr);
            double memWidth = ft.Width + 2 * MemLB.Padding.Left;
            if (ft.Height > maxH)
            {
                maxH = ft.Height;
            }

            // Window Size決定
            Height = 3 * 2 + maxH + 2 * 2;
            Width = 3 * 2 + wdWidth + spWidth + tmWidth + spWidth + memWidth;

            // Inner Grid Size 決定
            LbGrid.ColumnDefinitions[0].MaxWidth = LbGrid.ColumnDefinitions[0].MinWidth = wdWidth;
            LbGrid.ColumnDefinitions[1].MaxWidth = LbGrid.ColumnDefinitions[1].MinWidth = spWidth;
            LbGrid.ColumnDefinitions[2].MaxWidth = LbGrid.ColumnDefinitions[2].MinWidth = tmWidth;
            LbGrid.ColumnDefinitions[3].MaxWidth = LbGrid.ColumnDefinitions[3].MinWidth = spWidth;
            LbGrid.ColumnDefinitions[4].MaxWidth = LbGrid.ColumnDefinitions[4].MinWidth = memWidth;
        }

        // Size決定＆位置調整
        private void CalcSizeAndPos()
        {
            CalcSize();
            UpdateTimeAndMem();
            FormUtils.AdjustWindowPos(this, PosType, IgnoreTaskBar);
        }

        // Aboutの表示
        private void AboutMn_Click(object sender, RoutedEventArgs e)
        {
            aboutf fm = new aboutf();
            fm.Owner = this;
            fm.ShowDialog();
        }

        // マウスによる移動処理(PositionTypeがAnyの時のみ)
        private void label1_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (PosType != PositionType.Any) return;
            // postion typeがanyの時に限り移動可能
            if (label1.CaptureMouse())
            {
                sv_pt = PointToScreen(e.GetPosition(this));
                Cursor = Cursors.Hand;
            }
        }

        private void label1_MouseMove(object sender, MouseEventArgs e)
        {
            if (label1.IsMouseCaptured)
            {
                Point now_pt = PointToScreen(e.GetPosition(this));
                double dx = now_pt.X - sv_pt.X;
                dx = DeviceHelper.DeviceToWPF(dx, System.Windows.Forms.Orientation.Horizontal);
                double dy = now_pt.Y - sv_pt.Y;
                dy = DeviceHelper.DeviceToWPF(dy, System.Windows.Forms.Orientation.Vertical);
                Left += dx;
                Top += dy;
                sv_pt = now_pt;
            }
        }

        private void label1_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (label1.IsMouseCaptured)
            {
                label1.ReleaseMouseCapture();
                Cursor = Cursors.Arrow;
                ayaclockconfig.Instance.Left = (int)Left;
                ayaclockconfig.Instance.Top = (int)Top;
                ayaclockconfig.SaveConfig();
            }
        }

        // PositionTypeの設定
        private void PosTypeMn_Click(object sender, RoutedEventArgs e)
        {
            if (LeftTopMn.Equals(sender))
            {
                PosType = PositionType.LeftTop;
            }
            else if (LeftBottomMn.Equals(sender))
            {
                PosType = PositionType.LeftBottom;
            }
            else if (RightTopMn.Equals(sender))
            {
                PosType = PositionType.RightTop;
            }
            else if (RightBottomMn.Equals(sender))
            {
                PosType = PositionType.RightBottom;
            }
            else if (FreedomMn.Equals(sender))
            {
                PosType = PositionType.Any;
            }
        }

        // StayOnTopの設定
        private void StayOnTopMn_Click(object sender, RoutedEventArgs e)
        {
            StayOnTopMn.IsChecked = !StayOnTopMn.IsChecked;
            StayOnTop = StayOnTopMn.IsChecked;
        }

        // TaskBarを無視するかどうかの設定
        private void IgnoreTaskBarMn_Click(object sender, RoutedEventArgs e)
        {
            IgnoreTaskBarMn.IsChecked = !IgnoreTaskBarMn.IsChecked;
            IgnoreTaskBar = IgnoreTaskBarMn.IsChecked;
        }

        // Fontの設定 - underlineとstrikeはやってらんないのでパス
        private System.Drawing.Font MakeLabelToFont()
        {
            // font sizeを先に計算しておく - labelのFontSizeは1/96pt単位、FontのSizeは1/72pt単位
            double fontsz = System.Math.Round(label1.FontSize * 3 / 4);
            // 装飾の値を確定しておく
            System.Drawing.FontStyle fs = System.Drawing.FontStyle.Regular;
            if (label1.FontWeight.ToOpenTypeWeight() > 400) // check Bold 400=normal 700=bold
                fs = System.Drawing.FontStyle.Bold;
            if (label1.FontStyle == System.Windows.FontStyles.Italic) // check Italic
                fs |= System.Drawing.FontStyle.Italic;
            // 作成
            return new System.Drawing.Font(label1.FontFamily.ToString(),(float)fontsz,fs, System.Drawing.GraphicsUnit.Point);
        }

        private void FontMn_Click(object sender, RoutedEventArgs e)
        {
            // FontDialog作成
            System.Windows.Forms.FontDialog fd = new System.Windows.Forms.FontDialog();

            // FontDialog用にFont objectを仮作成 - フォント名・サイズ・Bold/Italicを設定
            fd.Font = MakeLabelToFont();

            // 色設定
            fd.ShowColor = true;
            fd.Color = System.Drawing.Color.FromArgb(Convert.ToInt32(DateLB.Foreground.ToString().Replace("#",""),16));

            // その他設定
            fd.FontMustExist = true;
            fd.AllowScriptChange = true;
            fd.AllowSimulations = true;
            fd.AllowVerticalFonts = false;

            if (fd.ShowDialog() != System.Windows.Forms.DialogResult.Cancel)
            {
                // Font名 - FontDialog.Font.NameをLabel1.FontFamilyに入れる 
                FontFamilyConverter ffc = new FontFamilyConverter();
                label1.FontFamily = (System.Windows.Media.FontFamily)ffc.ConvertFromString(fd.Font.Name);
                ayaclockconfig.Instance.FontName = fd.Font.Name;

                // Size - 1/72pt単位から1/96pt単位に変換して代入
                label1.FontSize = System.Math.Round(fd.Font.SizeInPoints * 4 / 3);
                ayaclockconfig.Instance.FontSize = label1.FontSize;

                // 色
                string argb = "#" + Convert.ToString(fd.Color.ToArgb(), 16);
                //BrushConverter bc = new BrushConverter();
                //label1.Foreground = (System.Windows.Media.Brush)bc.ConvertFrom(argb);
                ayaclockconfig.Instance.Color = fd.Color.ToArgb();

                // Bold
                if (fd.Font.Bold)
                    label1.FontWeight = System.Windows.FontWeight.FromOpenTypeWeight(700);
                else
                    label1.FontWeight = System.Windows.FontWeight.FromOpenTypeWeight(400);
                ayaclockconfig.Instance.Bold = fd.Font.Bold;

                // Italic
                if (fd.Font.Italic)
                    label1.FontStyle = System.Windows.FontStyles.Italic;
                else
                    label1.FontStyle = System.Windows.FontStyles.Normal;
                ayaclockconfig.Instance.Italic = fd.Font.Italic;

                // 設定
                FontSettingForLabels(argb);

                // 調整
                CalcSizeAndPos();

                // 保存
                ayaclockconfig.SaveConfig();
            }

        }

        // PopupMenuを開いた時、AdjustTimeの有効・無効を設定する
        private void PopupMenu_Opened(object sender, RoutedEventArgs e)
        {
            //AdjustTimeMn.IsEnabled = CheckSE();
            AdjustTimeMn.IsEnabled = true; // ver 0.02より常にtrue
        }

        // SeSystemtimePrivilege特権所有確認
        private bool CheckSE()
        {
            /*
            // 手抜きバージョン - Admin権限があればtrue
            AppDomain.CurrentDomain.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);
            WindowsPrincipal princ = (WindowsPrincipal)Thread.CurrentPrincipal;
            return princ.IsInRole(WindowsBuiltInRole.Administrator);
            */
            // 「ある程度」ちゃんとしたバージョン
            return SystemTimePrivilege.HaveSeSystemtimePrivilege();
        }

        // コントロールパネルを開く
        private void AdjustTimeMn_Click(object sender, RoutedEventArgs e)
        {
            System.Diagnostics.Process proc = new System.Diagnostics.Process();
            proc.StartInfo.FileName = "rundll32.exe";
            proc.StartInfo.Arguments = "shell32.dll,Control_RunDLL Timedate.cpl";
            proc.StartInfo.UseShellExecute = true;
            proc.StartInfo.Verb = "open";
            // xp 且つ 制限ユーザーの場合に限り、verbはRunAs
            System.OperatingSystem os = System.Environment.OSVersion;
            if (os.Version.Major==5 && os.Version.Minor==1 && !CheckSE())
            {
                proc.StartInfo.Verb = "runas";
            }
            proc.Start();
        }

        // メモリ表示量の単位設定
        private void MemUnitMn_Click(object sender, RoutedEventArgs e)
        {
            if (KByteMn.Equals(sender))
            {
                MemUnit = MemUnitType.KByte;
            }
            else if (MByteMn.Equals(sender))
            {
                MemUnit = MemUnitType.MByte;
            }
            else if (GByteMn.Equals(sender))
            {
                MemUnit = MemUnitType.GByte;
            }
            CalcSizeAndPos();
        }

        // メモリの区切り表示の有無
        private void CommaMn_Click(object sender, RoutedEventArgs e)
        {
            ShowComma = !ShowComma;
            CalcSizeAndPos();
        }

        // 曜日表示の設定
        private void WeekdayMn_Click(object sender, RoutedEventArgs e)
        {
            if (WeekDayNoneMn.Equals(sender))
                Weekday = WeekdayType.None;
            else if (En2ChMn.Equals(sender))
                Weekday = WeekdayType.Eng2ch;
            else if (En3ChMn.Equals(sender))
                Weekday = WeekdayType.Eng3ch;
            else if (KanjiMn.Equals(sender))
                Weekday = WeekdayType.Kanji;
            else if (OsWeekdayMn.Equals(sender))
                Weekday = WeekdayType.OsSetting;
            CalcSizeAndPos();
        }

        // 「秒」の表示の有無
        private void SecondMn_Click(object sender, RoutedEventArgs e)
        {
            ShowSecond = !ShowSecond;
            CalcSizeAndPos();
        }

        // TasktrayIcon表示の有無
        private void TasktrayMn_Click(object sender, RoutedEventArgs e)
        {
            ShowTasktrayIcon = !ShowTasktrayIcon;
        }

        // 終了するのでタスクトレイアイコンを非表示にする
        //   - これをしておかないとタスクトレイにアイコンが残ったままに...
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            ni.Visible = false;
        }

    }
}
