﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace AyaClock.NET
{
    public enum MemUnitType
    {
        KByte = 0,
        MByte,
        GByte
    }

    public enum WeekdayType
    {
        None = 0,
        Eng3ch,
        Eng2ch,
        Kanji,
        OsSetting
    }

    [Serializable()]
    public class ayaclockconfig
    {
        // classはpublicにしておかないとシリアライズの時点でエラーが発生する

        /// <summary>
        /// 位置情報
        /// </summary>
        private int _Left, _Top;
        private PositionType _PosType;
        private bool _StayOnTop,_IgnoreTaskBar;
        /// <summary>
        /// フォント情報
        /// </summary>
        private string _FontName;
        private double _FontSize;
        private int _Color;
        private bool _Bold, _Italic;
        /// <summary>
        /// メモリ表示
        /// </summary>
        private MemUnitType _MemUnit;
        private bool _ShowComma;
        /// <summary>
        /// 曜日表示
        /// </summary>
        private WeekdayType _Weekday;
        /// <summary>
        /// 秒表示
        /// </summary>
        private bool _ShowSecond;
        /// <summary>
        /// タスクトレイアイコン表示の有無
        /// </summary>
        private bool _ShowTasktrayIcon;

        public int Left
        {
            get { return _Left; }
            set { _Left = value; }
        }
        public int Top
        {
            get { return _Top; }
            set { _Top = value; }
        }
        public PositionType PosType
        {
            get { return _PosType; }
            set { _PosType = value; }
        }
        public bool StayOnTop
        {
            get { return _StayOnTop; }
            set { _StayOnTop = value; }
        }
        public bool IgnoreTaskBar
        {
            get { return _IgnoreTaskBar; }
            set { _IgnoreTaskBar = value; }
        }
        public string FontName
        {
            get { return _FontName; }
            set { _FontName = value; }
        }
        public double FontSize
        {
            get { return _FontSize; }
            set { if (value>0) _FontSize = value; }
        }
        public int Color
        {
            get { return _Color; }
            set { _Color = value; }
        }
        public bool Bold
        {
            get { return _Bold; }
            set { _Bold = value; }
        }
        public bool Italic
        {
            get { return _Italic; }
            set { _Italic = value; }
        }
        public MemUnitType MemUnit
        {
            get { return _MemUnit; }
            set { _MemUnit = value; }
        }
        public bool ShowComma
        {
            get { return _ShowComma; }
            set { _ShowComma = value; }
        }
        public WeekdayType Weekday
        {
            get { return _Weekday; }
            set { _Weekday = value; }
        }
        public bool ShowSecond
        {
            get { return _ShowSecond; }
            set { _ShowSecond = value; }
        }
        public bool ShowTasktrayIcon
        {
            get { return _ShowTasktrayIcon; }
            set { _ShowTasktrayIcon = value; }
        }

        // constructor
        public ayaclockconfig()
        {
            PosType = PositionType.LeftTop;
            Left = 0; Top = 0;
            StayOnTop = false;
            FontName = "メイリオ";
            FontSize = 12;
            Color = System.Drawing.Color.Black.ToArgb();
            Bold = false;
            Italic = false;
            MemUnit = MemUnitType.MByte;
            ShowComma = true;
            Weekday = WeekdayType.None;
            ShowSecond = false;
            _ShowTasktrayIcon = false;
        }

        // singleton instance
        [NonSerialized()]
        private static ayaclockconfig _instance;
        [System.Xml.Serialization.XmlIgnore]
        public static ayaclockconfig Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new ayaclockconfig();
                return _instance;
            }
            set { _instance = value; }
        }

        // load config
        public static void LoadConfig()
        {
            string path = GetConfigPath();
            FileStream fs=null;
            try
            {
                fs = new FileStream(path, FileMode.Open, FileAccess.Read);
                System.Xml.Serialization.XmlSerializer xs = new System.Xml.Serialization.XmlSerializer(typeof(ayaclockconfig));
                object obj = xs.Deserialize(fs);
                _instance = (ayaclockconfig)obj;
            }
            catch (FileNotFoundException)
            {
                // config.dataが無いのでなにもせん!
                return;
            }
            catch (Exception ex)
            {
                // なんぞのエラーが発生 > とりあえず無視
                MessageBox.Show(ex.Message, "load config");
            }
            finally
            {
                if (fs != null) fs.Close();
            }
        }

        // save config
        public static void SaveConfig()
        {
            string path = GetConfigPath();
            FileStream fs = null;
            try
            {
                fs = new FileStream(path,FileMode.Create,FileAccess.Write);
                System.Xml.Serialization.XmlSerializer xs = new System.Xml.Serialization.XmlSerializer(typeof(ayaclockconfig));
                xs.Serialize(fs, Instance);
            }
            catch (Exception ex)
            {
                // なんぞのエラーが発生
                MessageBox.Show(ex.Message,"save config");
            }
            finally
            {
                if (fs != null) fs.Close();
            }
        }

        // helper routines
        public static string GetConfigPath()
        {
            // save folderが無い場合は構築しておかねばならない！
            string result = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), // Roaming Folder
                                  "MarleySoft\\");
            if (!System.IO.Directory.Exists(result))
            {
                System.IO.Directory.CreateDirectory(result);
            }
            result = Path.Combine(result, "AyaClock.NET\\");
            if (!System.IO.Directory.Exists(result))
            {
                System.IO.Directory.CreateDirectory(result);
            }
            //
            result = Path.Combine(result, "config.data");
            return result;
        }
    }
}
